#include "stdafx.h"
#include "math.h"

const int PRO_ITER_LEN = 6;
const int PRO_DUR_LEN = 16;
const int PRO_VAL_LEN = 16;
const int PRO_FORMULA_LEN = 128;
const int PRO_REPEAT_LEN = 6;

double g_d1, g_d2;	// shorcut to return values from ProCalc since it is modal 
double g_bSetup=false;

class CVwProtocol : public CDialog { 
	public : 
		CProtocol m_oProtocol;			// Underlying object for Protocol data 
		RECT m_rctDisp;					// Boundary for Disp protocol window 
		RECT m_rctDispIter;				// Boundary for multiple protocol window 
		int m_iCurrSeg;					// Highlighted segment index 
		CString m_cLongName;			// file name - blank on New 

		CDlgControl m_btnSpin;
		CDlgControl m_btnAddSeg;
		CDlgControl m_btnDelSeg;

		CDlgEdit m_edtProDuration;
		CDlgEdit m_edtProIterations;
		CDlgCheck m_chkProRandom;
		CDlgEdit m_edtSegDuration;
		CDlgEdit m_edtSegV0;
		CDlgCheck m_chkSegV0Final;
		CDlgEdit m_edtSegV0Final;
		CDlgEdit m_edtSegV1;
		CDlgCheck m_chkSegV1Final;
		CDlgEdit m_edtSegV1Final;
		CDlgEdit m_edtSegFunction;
		CDlgCheck m_chkSegModifiable;
		CDlgEdit m_edtSegRepeat;

		CDlgControl m_lblDispMin;
		CDlgControl m_lblDispMax;
		CDlgControl m_lblIterMin;
		CDlgControl m_lblIterMax;

		CDlgControl m_picDisp;
		CDlgControl m_picDispAll;
		

		CVwProtocol() { 
			m_iID = IDD_PROTOCOL;
			m_cLongName.Set("");
			}

		void PrepareProtocol() { 
			CalculateDispWidths();
			m_oProtocol.CalculateIterWidths(663,0.0);
			CalculateMinMax();
			SetupProtocol();
			SetupSegment(0);
			}

		void GenProtocol( COptionsGenPulseProtocol * pOpt ) {
			//TODO - move this to CProtocol class
			int iDur, iTotalDur=0, iSeg=0;
			char bufDur[64], bufAmp[64];
			double dAmp;
			
			while( iSeg<200 && iTotalDur < pOpt->m_iTotalDuration ) { 
				if( (++iSeg)%2==0 ) { 
					iDur = int(RandomD( pOpt->m_iPulseDurMin, pOpt->m_iPulseDurMax, pOpt->m_bPulseExp ? -1 : 0 ));
					iDur = min( pOpt->m_iTotalDuration - iTotalDur, iDur );
					dAmp = RandomD( pOpt->m_dAmpMin, pOpt->m_dAmpMax, 0 );
					}
				else { 
					iDur = int(RandomD( pOpt->m_iSepDurMin, pOpt->m_iSepDurMax, pOpt->m_bSepExp ? -1 : 0 ));
					iDur = min( pOpt->m_iTotalDuration - iTotalDur, iDur );
					dAmp = pOpt->m_dBaseAmp;
					}
				sprintf( bufDur, "%d", iDur );

				if( pOpt->m_bRamp ) {
					sprintf( bufAmp, "ramp(0.5,%.3f,u,%.3f)", pOpt->m_dBaseAmp, pOpt->m_dBaseAmp );
					m_oProtocol.AddSegment( bufDur, dAmp, 1.0, false, 0.0, 1.0, false, bufAmp  );
					}
				else {
					sprintf( bufAmp, "%.3f", dAmp  );
					m_oProtocol.AddSegment( bufDur, 0.0, 1.0, false, 0.0, 1.0, false, bufAmp );
					}
				iTotalDur += iDur;
				}
				
			PrepareProtocol();
			}
			
		void NewProtocol() {			// Build a simple default protocol for new ones
			m_oProtocol.AddSegment( "500", 0.0, 0.0, false, false, 0.0, 0.0 );
			m_oProtocol.AddSegment( "1000", 5.0, 5.0, false, false, 0.0, 0.0 );
			m_oProtocol.AddSegment( "500", 0.0, 0.0, false, false, 0.0, 0.0 );
			PrepareProtocol();
			}

		void LoadProtocol( const char * pchFilename ) {
			if ( ! m_oProtocol.LoadFile(pchFilename) ) {
				MessageBox( m_hDlg, "Error loading protocol file", pchFilename, MB_OK);
				DestroyWindow( GetParent( m_hDlg ) );
				return ;
				}
			m_cLongName.Set( pchFilename );
			PrepareProtocol();
			}

		void InitializeControls( ) { 
			m_btnSpin.SetHandle( GetDlgItem( m_hDlg, IDC_SPINSEG ), "Move segment left or right" );
			m_btnAddSeg.SetHandle( GetDlgItem( m_hDlg, IDC_BTNADDSEG ), "Add a copy of the selected segment" );
			m_btnDelSeg.SetHandle( GetDlgItem( m_hDlg, IDC_BTNDELSEG ), "Delete selected segment" );
		
			m_edtProDuration.SetHandle( GetDlgItem( m_hDlg, IDC_PRO_DUR ), "Total duration per iteration" );
			m_edtProIterations.SetHandle( GetDlgItem( m_hDlg, IDC_PRO_ITER ), "Number of sweeps of all segments before repeating" );
			m_chkProRandom.SetHandle( GetDlgItem( m_hDlg, IDC_CHKRANDOM ), "Randomly sort iterations" );
			
			m_edtSegDuration.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_DUR ), "Duration for selected segment" );
			
			m_edtSegV0.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_V0 ), "User setable function parament - fixed value or starting value" );
			m_chkSegV0Final.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_CHKV0FINAL ), "Toggle usage of a range of values for 'u' parameter" );
			m_edtSegV0Final.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_V0FINAL ), "Ending value for 'u' parameter" );
			
			m_edtSegV1.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_V1 ), "Final value -or- user settable parameter for function" );
			m_chkSegV1Final.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_CHKV1FINAL ), "Toggle usage of a range of values for 'v' parameter" );
			m_edtSegV1Final.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_V1FINAL ), "Ending value for 'v' parameter" );

			m_edtSegFunction.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_FUNCTION ), "User defined function of i=iteration[0..1] t=pos. in segment[0..1] and u,v set above" );
			m_chkSegModifiable.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_MODIFIABLE ), "Edit duration and u,v if applicable in the scope during generation" );
			m_edtSegRepeat.SetHandle( GetDlgItem( m_hDlg, IDC_SEG_REPEAT ), "Repetitions within segment of specified function");
			
			m_lblDispMax.SetHandle( GetDlgItem( m_hDlg, IDC_VMAX ) );
			m_lblDispMin.SetHandle( GetDlgItem( m_hDlg, IDC_VMIN ) );
			m_lblIterMax.SetHandle( GetDlgItem( m_hDlg, IDC_LBLMAX ) );
			m_lblIterMin.SetHandle( GetDlgItem( m_hDlg, IDC_LBLMIN ) );
			
			m_picDisp.SetHandle( GetDlgItem( m_hDlg, IDC_DISP ) );
			m_picDispAll.SetHandle( GetDlgItem( m_hDlg, IDC_DISPALL ) );
			RECT rct;
			m_picDisp.GetControlBounds( &rct );
			CreateTip( rct.left, rct.top, rct.right, rct.bottom, "Click segment to select" );
			m_picDispAll.GetControlBounds( &rct );
			CreateTip( rct.left, rct.top, rct.right, rct.bottom, "display of all iterations (if <= 100)" );

			m_edtProDuration.SetTextLimit( PRO_DUR_LEN );
			m_edtProIterations.SetTextLimit( PRO_ITER_LEN );
			m_edtSegDuration.SetTextLimit( PRO_DUR_LEN );
			m_edtSegV0.SetTextLimit( PRO_VAL_LEN );
			m_edtSegV1.SetTextLimit( PRO_VAL_LEN );
			m_edtSegV0Final.SetTextLimit( PRO_VAL_LEN );
			m_edtSegV1Final.SetTextLimit( PRO_VAL_LEN );
			m_edtSegFunction.SetTextLimit( PRO_FORMULA_LEN );
			m_edtSegRepeat.SetTextLimit( PRO_REPEAT_LEN );

			m_picDisp.GetControlBounds( &m_rctDisp );
			m_picDispAll.GetControlBounds( &m_rctDispIter );
			}

		void CalculateDispWidths(  ) { 
			m_oProtocol.CalculateDispWidths( m_rctDisp.right-m_rctDisp.left );
			}

		void CalculateMinMax() { 
			double dVal, aVal[4];	// i,t,u,v
			int iSeg, iX;
			CProtocolSegment * pSeg;
			
			//-- calc min and max 
			m_oProtocol.m_dYMax=0.0001;
			m_oProtocol.m_dYMin=0.0;
			for( int iIter=0; iIter<m_oProtocol.m_iIterations; ++iIter ) {
				aVal[0]=double(iIter)/max(1,m_oProtocol.m_iIterations-1);
				for( iSeg=0; iSeg<m_oProtocol.m_aSeg.Count(); ++iSeg ) {
					pSeg = m_oProtocol.Segment(iSeg);
					pSeg->Parse();
					pSeg->CalcAVal23( aVal, iIter, m_oProtocol.m_iIterations);
					for( iX=0; iX<pSeg->m_iIterWidth; ++iX ) {
						pSeg->CalcAval1( aVal, iX, pSeg->m_iIterWidth );
						dVal = pSeg->m_oExpr.Evaluate( aVal );
						m_oProtocol.m_dYMax = max( m_oProtocol.m_dYMax, dVal );
						m_oProtocol.m_dYMin = min( m_oProtocol.m_dYMin, dVal );
						}
					}
				}
			m_oProtocol.m_dYMax *= 1.1;
			m_oProtocol.m_dYMin *= 1.1;
			
			//-- Y axis labels 			
			m_lblDispMin.SetTextDouble( m_oProtocol.m_dYMin, "%.4f" );
			m_lblDispMax.SetTextDouble( m_oProtocol.m_dYMax, "%.4f" );
			m_lblIterMin.SetTextDouble( m_oProtocol.m_dYMin, "%.4f" );
			m_lblIterMax.SetTextDouble( m_oProtocol.m_dYMax, "%.4f" );
			} 
		
		void CauseRedraw( bool bRecalcWidths ) { 			
			if( bRecalcWidths ) {
				CalculateDispWidths();
				SetDlgItemInt( m_hDlg, IDC_PRO_DUR, m_oProtocol.Duration(), FALSE );
				}
			CalculateMinMax();
			InvalidateRect( m_hDlg, &m_rctDisp, true );
			InvalidateRect( m_hDlg, &m_rctDispIter, true );
			}
			
		bool SetupSegment( int iSeg ) { 
			g_bSetup=true;
			m_iCurrSeg=iSeg;
			CProtocolSegment * pSeg = m_oProtocol.Segment(iSeg);
			//-- Duration 
			m_edtSegDuration.SetText( pSeg->m_cDuration.ConstBuffer() );
			//-- V0, V1
			m_edtSegV0.SetTextDouble( pSeg->GetV0() );
			m_edtSegV1.SetTextDouble( pSeg->GetV1() );
			m_chkSegV0Final.SetCheck( pSeg->m_bV0Range );
			m_chkSegV1Final.SetCheck( pSeg->m_bV1Range );
			m_edtSegV0Final.SetTextDouble( pSeg->GetV0Final() );
			m_edtSegV1Final.SetTextDouble( pSeg->GetV1Final() );

			//-- Modifiable
			m_chkSegModifiable.SetCheck( pSeg->m_bModifiable );
			//-- Repear Count
			m_edtSegRepeat.SetTextInt( pSeg->m_iRepeatCount );
			//-- user function 
			m_edtSegFunction.SetText( pSeg->m_cFunction.Buffer() );
			
			g_bSetup=false;
			return true;
			}
			
		bool SetupProtocol( ) { 
			//-- Duration Total
			SetDlgItemInt( m_hDlg, IDC_PRO_DUR, m_oProtocol.Duration(), FALSE );
			
			//-- Iterations
			SetDlgItemInt( m_hDlg, IDC_PRO_ITER, max(1,min(999999,m_oProtocol.m_iIterations)), FALSE );

			//-- Random 
			m_chkProRandom.SetCheck( m_oProtocol.m_bRandom );

			return true;
			}
			
		void DrawProtocol(HDC hdc) { 
			CProtocolSegment * pSeg;
			int iSeg, iX0=0, iY, iX;
			int iSumDur = m_oProtocol.Duration();
			double dVal, aVal[4];	// i,t,u,v

			HPEN penRed = CreatePen( 0, 0, RGB(255,0,0) );
			HPEN penGray = CreatePen( 0, 0, RGB(64,64,64) );
			HPEN penBlack = CreatePen( 0, 0, RGB(0,0,0) );
			HPEN penBlue = CreatePen( 0, 0, RGB(0,0,255) );
			HPEN penWas = (HPEN) SelectObject( hdc, penGray );
			
			HBRUSH brWhite = (HBRUSH) GetStockObject(WHITE_BRUSH);
			HBRUSH brGray = (HBRUSH) GetStockObject(LTGRAY_BRUSH);
			HBRUSH brWas = (HBRUSH) SelectObject( hdc, brGray );
			
			// segment frames
			iX0=m_rctDisp.left;
			for( iSeg=0; iSeg<m_oProtocol.m_aSeg.Count(); ++iSeg ) {
				pSeg = m_oProtocol.Segment(iSeg);

				if( iSeg == m_iCurrSeg ) {
					SelectObject( hdc, brWhite );
					SelectObject( hdc, penBlue );
					}
				else { 
					SelectObject( hdc, brGray );
					SelectObject( hdc, penGray );
					}
				Rectangle( hdc, iX0, m_rctDisp.top, iX0+pSeg->m_iDispWidth, m_rctDisp.bottom );
				iX0+=pSeg->m_iDispWidth;
				}

			// data trace
			bool bError=false;
			SelectObject( hdc, penBlack );
			iX0=m_rctDisp.left;
			aVal[0]=0.0;
			IntersectClipRect( hdc, m_rctDisp.left, m_rctDisp.top, m_rctDisp.right, m_rctDisp.bottom );

			int iSegs = m_oProtocol.m_aSeg.Count();
			for( iSeg=0; iSeg<iSegs; ++iSeg ) {
				pSeg = m_oProtocol.Segment(iSeg);
				pSeg->CalcAVal23( aVal, 0, m_oProtocol.m_iIterations);
				
				for( iX=0; iX<pSeg->m_iDispWidth; ++iX ) {
					// calculate Y value and pixel pos
					pSeg->CalcAval1( aVal, iX, pSeg->m_iDispWidth );
					dVal = pSeg->m_oExpr.Evaluate( aVal, &bError );
					iY = m_rctDisp.bottom - int( 0.5 + (dVal-m_oProtocol.m_dYMin) / m_oProtocol.Range()*(m_rctDisp.bottom-m_rctDisp.top) );
					// if evaluate err'd, draw in red
					if( bError )			
						SelectObject( hdc, penRed ); 
					// plot as line segment 
					if( iSeg==0 && iX==0 ) 
						MoveToEx( hdc, iX0+iX, iY, NULL );
					else 
						LineTo( hdc, iX0+iX, iY );
					// after error return to previous pen 
					if( bError )			
						SelectObject( hdc, iSeg == m_iCurrSeg ? penBlue : penGray);
					}
				
				iX0+=pSeg->m_iDispWidth;
				}
			SelectClipRgn( hdc, NULL );
			
			//-- Multiple Iterations 
			if( m_oProtocol.m_iIterations<=100 ) { 
				//-- draw all protocols
				m_oProtocol.CreateRandomMap();
				int iIterDuration, iAccumDuration=0, iTotalDur = m_oProtocol.TotalDurations( );
				int iIterWidth, iAccumWidth=0, iTotalWidth = m_rctDispIter.right - m_rctDispIter.left;
				
				IntersectClipRect( hdc, m_rctDispIter.left, m_rctDispIter.top, m_rctDispIter.right, m_rctDispIter.bottom );
				iX0=m_rctDispIter.left;

				for( int iIter=0; iIter<m_oProtocol.m_iIterations; ++iIter ) {
					//-- duration calc and accumulators
					aVal[0] = m_oProtocol.IterationMap( iIter );
					double dIter = m_oProtocol.Iteration( iIter );
					iIterDuration = m_oProtocol.Duration( dIter );
					iIterWidth = MulDiv( iTotalWidth, iIterDuration+iAccumDuration, iTotalDur ) - iAccumWidth;
					iAccumDuration += iIterDuration;
					iAccumWidth += iIterWidth;
					//-- line separating iterations
					if( iIter > 0 ) {
						MoveToEx( hdc, iX0, m_rctDispIter.top, NULL );
						LineTo( hdc, iX0, m_rctDispIter.bottom );
						}
					//-- each segment in iteration
					m_oProtocol.CalculateIterWidths( iIterWidth, dIter );
					for( iSeg=0; iSeg<m_oProtocol.m_aSeg.Count(); ++iSeg ) {
						pSeg = m_oProtocol.Segment(iSeg);
						pSeg->CalcAVal23( aVal, iIter, m_oProtocol.m_iIterations);

						for( iX=0; iX<pSeg->m_iIterWidth; ++iX ) {
							pSeg->CalcAval1( aVal, iX, pSeg->m_iIterWidth );
							dVal = pSeg->m_oExpr.Evaluate( aVal );
							iY = m_rctDispIter.bottom - int( 0.5 + (dVal-m_oProtocol.m_dYMin) / m_oProtocol.Range()*(m_rctDispIter.bottom-m_rctDispIter.top) );
							if( iX==0 ) 
								MoveToEx( hdc, iX0+iX, iY, NULL );
							else 
								LineTo( hdc, iX0+iX, iY );
							}
						iX0+=pSeg->m_iIterWidth;
						}
					}
				SelectClipRgn( hdc, NULL );
				}
			
			//-- Cleanup 
			SelectObject( hdc, brWas );
			//DeleteObject( brWhite );		// unneccesary but not harmful
			//DeleteObject( brGray );		// unneccesary but not harmful
			SelectObject( hdc, penWas );
			DeleteObject( penRed );
			DeleteObject( penGray );
			DeleteObject( penBlack );
			DeleteObject( penBlue );
			}				
	};

//-- Protocol view message handler
BOOL CALLBACK ProtocolDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	PAINTSTRUCT ps;
	int wmId, wmEvent;
	CVwProtocol * pVwPro;
	
	switch( iMsg ) { 
		case WM_INITDIALOG :
			pVwPro = new CVwProtocol;
			pVwPro->m_hDlg=hDlg;
			SetWindowLong( GetParent( hDlg ), 0,  (long) pVwPro ) ;
			pVwPro->InitializeControls();
			return TRUE;
			
		case WM_PRO_GEN :		// User message for displaying generated protocol
			((CVwProtocol *) GetWindowLong( GetParent( hDlg ), 0 ))->GenProtocol( (COptionsGenPulseProtocol *)lParam );		
			return TRUE;
			
		case WM_PRO_LOAD :		// User message for menu item to load file 
			((CVwProtocol *) GetWindowLong( GetParent( hDlg ), 0 ))->LoadProtocol( (char *)lParam );
			return TRUE;

		case WM_PRO_NEW :		// user message for menu item to create a new protocol
			((CVwProtocol *) GetWindowLong( GetParent( hDlg ), 0 ))->NewProtocol();		
			return TRUE;

		case WM_NOTIFY :		// spin control (up down control) to move segment left or right
			wmId    = LOWORD(wParam); 
			pVwPro = (CVwProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			//----- Move segment left or right
			if( wmId == IDC_SPINSEG ) { 
				int iPos, iDelta = ((LPNMUPDOWN) lParam)->iDelta;
				// move to the right (-1)
				if( iDelta == -1 && (iPos=(pVwPro->m_iCurrSeg+1))<pVwPro->m_oProtocol.m_aSeg.Count() )
					pVwPro->m_oProtocol.MoveSegment( pVwPro->m_iCurrSeg, pVwPro->m_iCurrSeg+1 );
				// move to the left (1)
				else if( iDelta == 1 && (iPos=(pVwPro->m_iCurrSeg-1)) >=0 )
					pVwPro->m_oProtocol.MoveSegment( pVwPro->m_iCurrSeg, pVwPro->m_iCurrSeg-1 );
				// something else - ignore
				else 
					return FALSE;
				// update the rest
				pVwPro->SetupSegment( iPos );
				pVwPro->CauseRedraw( true );				
					
				return TRUE;
				}
			break;
			
		case WM_COMMAND : {
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam);
			pVwPro = (CVwProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );

			//----- Random flag changed
			if( wmId == IDC_CHKRANDOM ) {
				pVwPro->m_oProtocol.m_bRandom = pVwPro->m_chkProRandom.GetCheck();
				pVwPro->CauseRedraw(true);
				return TRUE;
				}

			//----- Iterations changed
			if( wmId == IDC_PRO_ITER ) {
				if( wmEvent==EN_CHANGE ) {
					pVwPro->m_oProtocol.m_iIterations = pVwPro->m_edtProIterations.GetTextInt();
					pVwPro->CauseRedraw(true);
					return TRUE;
					} 
				}

			//----- Duration changed
			if( wmId == IDC_SEG_DUR ) {
				if( wmEvent==EN_CHANGE ) {
					CString cDur;
					pVwPro->m_edtSegDuration.GetText(&cDur);				
					pVwPro->m_oProtocol.Segment(pVwPro->m_iCurrSeg)->SetDuration( cDur.ConstBuffer() );
					pVwPro->CauseRedraw(true);
					return TRUE;
					} 
				}

			if( ! g_bSetup ) { 
				//----- V0 changed
				if( (( wmId == IDC_SEG_V0 || wmId==IDC_SEG_V0FINAL ) && wmEvent==EN_CHANGE ) 
					|| wmId==IDC_SEG_CHKV0FINAL ) {
					pVwPro->m_oProtocol.Segment(pVwPro->m_iCurrSeg)->SetV0(
						pVwPro->m_edtSegV0.GetTextDouble(),
						pVwPro->m_chkSegV0Final.GetCheck(),
						pVwPro->m_edtSegV0Final.GetTextDouble()
						);
					pVwPro->CauseRedraw(false);
					return TRUE;
					}

				//----- V1 changed
				if( (( wmId == IDC_SEG_V1 || wmId==IDC_SEG_V1FINAL ) && wmEvent==EN_CHANGE ) 
					|| wmId==IDC_SEG_CHKV1FINAL ) {
					pVwPro->m_oProtocol.Segment(pVwPro->m_iCurrSeg)->SetV1(
						pVwPro->m_edtSegV1.GetTextDouble(),
						pVwPro->m_chkSegV1Final.GetCheck(),
						pVwPro->m_edtSegV1Final.GetTextDouble()
						);
					pVwPro->CauseRedraw(false);
					return TRUE;
					}
				}

			//----- Modifiable changed
			if( wmId == IDC_SEG_MODIFIABLE ) {
				pVwPro->m_oProtocol.Segment(pVwPro->m_iCurrSeg)->m_bModifiable = pVwPro->m_chkSegModifiable.GetCheck();
				pVwPro->CauseRedraw(false);
				return TRUE;
				}

			//----- Segment repeat count changed
			if( wmId==IDC_SEG_REPEAT ) {
				pVwPro->m_oProtocol.Segment(pVwPro->m_iCurrSeg)->m_iRepeatCount = pVwPro->m_edtSegRepeat.GetTextInt();
				pVwPro->CauseRedraw(false);
				return TRUE;
				}


			//----- Function changed
			if( wmId == IDC_SEG_FUNCTION ) {
				if( wmEvent==EN_CHANGE ) {
					CString cFunc;
					pVwPro->m_edtSegFunction.GetText(&cFunc);
					pVwPro->m_oProtocol.Segment(pVwPro->m_iCurrSeg)->m_cFunction.Set( cFunc.Buffer() );
					pVwPro->CauseRedraw(false);
					return TRUE;
					} 
				}

			//----- Save Protocol
			if( wmId == IDC_BTNSAVE ) { 
				char bufName[_MAX_PATH+1]="";
				if( pVwPro->m_cLongName.Length()!=0 && pVwPro->m_cLongName.Length()<_MAX_PATH ) 
					sprintf( bufName, "%s", pVwPro->m_cLongName.ConstBuffer() );
					
				//-- select the file to save
				if( ! GetSaveFileName( bufName, 
								"Protocol Files (*.qpr)\0*.qpr\0All Files (*.*)\0*.*\0\0",
								"qpr", 
								GetParent(hDlg), NULL,
								& g_oCfgs.m_oSettings.m_cProtoPath ) )
					return TRUE;
				pVwPro->m_cLongName.Set(bufName);
					
				if( pVwPro->m_oProtocol.SaveFile( pVwPro->m_cLongName.Buffer() ) ) 
					DestroyWindow( GetParent( hDlg ) );
				return TRUE;
				}

			//----- Cancel protocol edit 
			if( wmId == IDC_BTNCANCEL ) { 
				DestroyWindow( GetParent( hDlg ) );
				return TRUE;
				}

			//----- Delete the current segment ( unless it is the only segment )
			if( wmId == IDC_BTNDELSEG ) { 
				if( pVwPro->m_oProtocol.m_aSeg.Count()>1 )  { 
					int iCurr=pVwPro->m_iCurrSeg;
					pVwPro->m_oProtocol.DelSegment( iCurr );
					pVwPro->SetupSegment( max(0,iCurr-1) );
					pVwPro->CauseRedraw(true);				
					}
				return TRUE;
				}

			//----- Add a new segment after the current segment 
			if( wmId == IDC_BTNADDSEG ) { 
				int iCurr=pVwPro->m_iCurrSeg;
				CProtocolSegment * pSeg = pVwPro->m_oProtocol.Segment(iCurr);
				if( pSeg!=NULL ) {
					pVwPro->m_oProtocol.AddSegment( pSeg->m_cDuration.ConstBuffer(), pSeg->GetV0(), pSeg->GetV1(), 
							pSeg->m_bV0Range, pSeg->m_bV1Range, pSeg->GetV0Final(), pSeg->GetV1Final(), 
							pSeg->m_cFunction.Buffer(), pSeg->m_bModifiable, pSeg->m_iRepeatCount, iCurr+1 );
					pVwPro->SetupSegment( iCurr+1 );
					pVwPro->CauseRedraw(true);		
					}

				return TRUE;
				}
			}
			
			return FALSE;
			
		case WM_LBUTTONDOWN : {
			pVwPro = (CVwProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			int iX = LOWORD(lParam); 
			int iY = HIWORD(lParam);
			
			POINT p;
			p.x=LOWORD(lParam);
			p.y=HIWORD(lParam);
			//----- Click in Disp area 
			if( PtInRect( &pVwPro->m_rctDisp, p ) ) { 
				int iSeg = pVwPro->m_oProtocol.SegmentFromPos(p.x-pVwPro->m_rctDisp.left );
				if( iSeg!=-1 && iSeg!=pVwPro->m_iCurrSeg ) { 
					pVwPro->SetupSegment(iSeg);
					pVwPro->CauseRedraw(false);
					}
				}
			}
			break;
			
		case WM_PAINT : {
			HDC hdc = BeginPaint(hDlg, &ps);
							
			//----- Manual Drawing 
			pVwPro = (CVwProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			pVwPro->DrawProtocol( hdc );
			
			//----- Done
			EndPaint(hDlg, &ps);
			}
			return false;
			
		case WM_DESTROY :
			pVwPro = (CVwProtocol *) GetWindowLong(  GetParent( hDlg ), 0 );
			delete pVwPro;
			SetWindowLong( GetParent( hDlg ), 0, 0 );
			WindowListRemove( hDlg );
			return TRUE;

		default : 
			break;
		}
		
	return FALSE;
	} 
